import types from '@babel/types';

function isJSXAttribute(attr, name) {
  return types.isJSXAttribute(attr) && attr.name.name === name;
}
function findAttr(node, name) {
  const attr = node.openingElement.attributes.find((a) => isJSXAttribute(a, name));
  return attr && attr.value;
}
function index({ types: t }) {
  return {
    name: "react directive",
    visitor: {
      JSXElement(path) {
        const rif = findAttr(path.node, "r-if");
        const rfor = findAttr(path.node, "r-for");
        const { openingElement, closingElement, children, selfClosing } = path.node;
        openingElement.attributes = openingElement.attributes.filter((attr) => !(isJSXAttribute(attr, "r-if") || isJSXAttribute(attr, "r-for")));
        if (rfor && t.isBinaryExpression(rfor.expression, { operator: "in" })) {
          let forValue, forKey, forArray;
          const v_rfor = rfor.expression;
          if (t.isIdentifier(v_rfor.right)) {
            forArray = v_rfor.right;
          }
          if (t.isSequenceExpression(v_rfor.left)) {
            [forValue, forKey] = v_rfor.left.expressions;
          } else if (t.isIdentifier(v_rfor.left)) {
            forValue = v_rfor.left;
            forKey = t.identifier("index");
          } else {
            return;
          }
          if (forArray && forValue && forKey) {
            const key = findAttr(path.node, "key");
            if (!key) {
              openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier("key"), t.jsxExpressionContainer(forKey)));
            }
            let forItem = path.node;
            if (rif) {
              forItem = t.conditionalExpression(rif.expression, t.jsxElement(openingElement, closingElement, children, selfClosing), t.nullLiteral());
            }
            path.replaceWith(t.expressionStatement(t.callExpression(t.memberExpression(forArray, t.identifier("map")), [t.arrowFunctionExpression([forValue, forKey], forItem)])));
          }
        } else if (rif) {
          path.replaceWith(t.jsxExpressionContainer(t.conditionalExpression(rif.expression, t.jsxElement(openingElement, closingElement, children, selfClosing), t.nullLiteral())));
        }
      }
    }
  };
}

export { index as default };
